from django.core.exceptions import ValidationError
from django_filters import rest_framework as filters
from drf_spectacular.utils import extend_schema, OpenApiParameter
from rest_framework import viewsets, status
from rest_framework.response import Response

from greg.api.filters import PersonFilter, RoleFilter, IdentityFilter
from greg.api.pagination import PrimaryKeyCursorPagination
from greg.api.serializers.person import (
    PersonSerializer,
    IdentitySerializer,
)
from greg.api.serializers.role import RoleSerializer, RoleWriteSerializer
from greg.models import Person, Role, Identity


class PersonViewSet(viewsets.ModelViewSet):
    """Person API"""

    queryset = Person.objects.all().order_by("id")
    serializer_class = PersonSerializer
    pagination_class = PrimaryKeyCursorPagination
    lookup_field = "id"
    filter_backends = (filters.DjangoFilterBackend,)
    filterset_class = PersonFilter

    @extend_schema(
        parameters=[
            OpenApiParameter(
                name="verified",
                description="Only include verified or only not verified. When nothing is set, "
                "both verified and not verified are returned",
                required=False,
                type=bool,
            ),
            OpenApiParameter(
                name="active",
                description="If try only include persons with an active role",
                required=False,
                type=bool,
            ),
        ]
    )
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)


class RoleViewSet(viewsets.ModelViewSet):
    """Person role API"""

    queryset = Role.objects.all().order_by("id")
    pagination_class = PrimaryKeyCursorPagination
    filter_backends = (filters.DjangoFilterBackend,)
    filterset_class = RoleFilter
    lookup_field = "id"

    def get_serializer_class(self):
        """
        Fetch different serializer depending on http method.

        To return the complete orgunit object for GET we use a different serializer.
        """
        if self.request.method in ("POST", "PATCH"):
            return RoleWriteSerializer
        return RoleSerializer

    def get_queryset(self):
        qs = self.queryset
        if not self.kwargs:
            return qs.none()
        person_id = self.kwargs["person_id"]
        person_role_id = self.kwargs.get("id")
        qs = qs.filter(person_id=person_id)
        if person_role_id:
            qs = qs.filter(id=person_role_id)
        return qs

    def perform_create(self, serializer):
        person_id = self.kwargs["person_id"]

        if person_id is None:
            # Should not happen, the person ID is part of the API path
            raise ValidationError("No person id")

        serializer.save(person_id=person_id)


class IdentityViewSet(viewsets.ModelViewSet):
    """
    Person identity API
    """

    queryset = Identity.objects.all().order_by("id")
    serializer_class = IdentitySerializer
    pagination_class = PrimaryKeyCursorPagination
    filter_backends = (filters.DjangoFilterBackend,)
    filterset_class = IdentityFilter
    # This is set so that the id parameter in the path of the URL is used for looking up objects
    lookup_url_kwarg = "id"

    def get_queryset(self):
        qs = self.queryset
        if not self.kwargs:
            return qs.none()
        person_id = self.kwargs["person_id"]
        qs = qs.filter(person_id=person_id)
        return qs

    def create(self, request, *args, **kwargs):
        # Want to get the person id which is part of the API path and then
        # include this with the data used to create the identity for the person
        person_id = self.kwargs["person_id"]

        if person_id is None:
            # Should not happen, the person ID is part of the API path
            raise ValidationError("No person id")

        input_data = request.data.copy()
        input_data["person"] = person_id

        serializer = self.get_serializer(data=input_data)
        serializer.is_valid(raise_exception=True)

        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(
            serializer.data, status=status.HTTP_201_CREATED, headers=headers
        )
