import datetime
from django.utils import timezone
import pytest

from rest_framework import status
from rest_framework.reverse import reverse
from rest_framework.test import APIClient

from greg.models import InvitationLink, Person, Identity


@pytest.mark.django_db
def test_post_invite(client):
    """Forbid access with bad invitation link uuid"""
    response = client.post(reverse("gregui-v1:invite-verify"), data={"uuid": "baduuid"})
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_post_invite_ok(client, invitation_link):
    """Access okay with valid invitation link"""
    response = client.post(
        reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid}
    )
    assert response.status_code == status.HTTP_200_OK


@pytest.mark.django_db
def test_post_invite_expired(client, invitation_link_expired):
    """Forbid access with expired invite link"""
    response = client.post(
        reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link_expired.uuid}
    )
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_post_missing_invite_id(client):
    """Forbid access if no id provided."""
    response = client.post(reverse("gregui-v1:invite-verify"))
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_get_invited_info_no_session(client, invitation_link):
    """Forbid access if invite expired after session was created."""
    # get a session
    client.post(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})
    # expire the invitation link
    invitation_link.expire = timezone.now() - datetime.timedelta(days=1)
    invitation_link.save()
    # fail to get info because session has expired
    response = client.get(reverse("gregui-v1:invited-info"))
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_get_invited_info_session_okay(
    client, invited_person, sponsor_foo_data, role_type_foo, unit_foo
):
    person, invitation_link = invited_person
    # get a session
    client.post(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})
    # Get info
    response = client.get(reverse("gregui-v1:invited-info"))
    assert response.status_code == status.HTTP_200_OK
    data = response.json()
    assert data.get("person") == dict(
        first_name=person.first_name,
        last_name=person.last_name,
        email=person.private_email.value,
        mobile_phone=None,
        fnr=None,
        passport=None,
        feide_id=None,
        date_of_birth=None,
    )
    assert data.get("sponsor") == dict(
        first_name=sponsor_foo_data["first_name"],
        last_name=sponsor_foo_data["last_name"],
    )
    assert data.get("role") == dict(
        start=None,
        end="2050-10-15",
        comments="",
        ou_name_en=unit_foo.name_en,
        ou_name_nb=unit_foo.name_nb,
        role_name_en=role_type_foo.name_en,
        role_name_nb=role_type_foo.name_nb,
    )


@pytest.mark.django_db
def test_get_invited_info_expired_link(
    client, invitation_link, invitation_expired_date
):
    # Get a session while link is valid
    client.get(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})
    # Set expire link to expire long ago
    invlink = InvitationLink.objects.get(uuid=invitation_link.uuid)
    invlink.expire = invitation_expired_date
    invlink.save()
    # Make a get request that should fail because invite expired after login, but
    # before get to userinfo
    response = client.get(reverse("gregui-v1:invited-info"))
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_invited_guest_can_post_information(client: APIClient, invited_person):
    person, invitation_link = invited_person
    # get a session
    client.post(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})

    person = invitation_link.invitation.role.person
    assert person.private_mobile is None

    # post updated info to confirm from guest
    new_email = "private@example.org"
    new_phone = "+4790000000"
    data = {"person": {"mobile_phone": new_phone, "email": new_email}}

    response = client.post(
        reverse("gregui-v1:invited-info"),
        data,
        format="json",
    )
    assert response.status_code == status.HTTP_200_OK

    # Check that the object was updated in the database
    assert Person.objects.count() == 1
    assert person.private_mobile.value == new_phone
    assert person.private_email.value == new_email


@pytest.mark.django_db
def test_post_invited_info_expired_session(
    client, invitation_link, invitation_expired_date
):
    # get a session
    client.post(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})
    # Set expire link to expire long ago
    invlink = InvitationLink.objects.get(uuid=invitation_link.uuid)
    invlink.expire = invitation_expired_date
    invlink.save()
    # post updated info to confirm from guest, should fail because of expired
    # invitation link
    response = client.post(reverse("gregui-v1:invited-info"))
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_post_invited_info_deleted_inv_link(client, invitation_link):
    # get a session
    client.post(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})
    # Delete link
    invlink = InvitationLink.objects.get(uuid=invitation_link.uuid)
    invlink.delete()
    # post updated info to confirm from guest, should fail because of removed
    # invitation link
    response = client.post(reverse("gregui-v1:invited-info"))
    assert response.status_code == status.HTTP_403_FORBIDDEN


@pytest.mark.django_db
def test_post_invited_info_invalid_national_id_number(client, invited_person):
    person, invitation_link = invited_person
    data = {
        "person": {
            "mobile_phone": "+4707543001",
            "email": "test@example.com",
            "fnr": "123",
        }
    }
    url = reverse("gregui-v1:invited-info")

    assert person.fnr is None

    session = client.session
    session["invite_id"] = str(invitation_link.uuid)
    session.save()

    response = client.post(url, data, format="json")

    # The request should fail and the fnr-property should stay unchanged
    assert response.status_code == status.HTTP_400_BAD_REQUEST
    person.refresh_from_db()
    assert person.fnr is None


@pytest.mark.django_db
def test_post_invited_info_valid_national_id_number(client, invited_person):
    person, invitation_link = invited_person
    fnr = "11120618212"
    data = {
        "person": {
            "mobile_phone": "+4797543992",
            "email": "test@example.com",
            "fnr": fnr,
        }
    }
    url = reverse("gregui-v1:invited-info")

    assert person.fnr is None

    session = client.session
    session["invite_id"] = str(invitation_link.uuid)
    session.save()

    response = client.post(url, data, format="json")

    assert response.status_code == status.HTTP_200_OK
    person.refresh_from_db()
    assert person.fnr.value == fnr


@pytest.mark.django_db
def test_email_update(client, invited_person):
    person, invitation_link = invited_person
    email_test = person.private_email.value
    url = reverse("gregui-v1:invited-info")

    person.private_email.refresh_from_db()
    assert person.private_email.value == email_test

    session = client.session
    session["invite_id"] = str(invitation_link.uuid)
    session.save()

    data = {"person": {"mobile_phone": "+4797543992", "email": "test2@example.com"}}
    response = client.post(url, data, format="json")
    assert response.status_code == status.HTTP_200_OK

    person.private_email.refresh_from_db()
    assert person.private_email.value == "test2@example.com"


@pytest.mark.django_db
def test_register_passport(client, invited_person):
    person, invitation_link = invited_person
    passport_information = "EN-123456789"
    data = {"person": {"mobile_phone": "+4797543992", "passport": passport_information}}
    url = reverse("gregui-v1:invited-info")

    session = client.session
    session["invite_id"] = str(invitation_link.uuid)
    session.save()

    assert (
        Identity.objects.filter(
            person__id=person.id, type=Identity.IdentityType.PASSPORT_NUMBER
        ).count()
        == 0
    )

    response = client.post(url, data, format="json")

    assert response.status_code == status.HTTP_200_OK
    person.refresh_from_db()

    registered_passport = Identity.objects.filter(
        person__id=person.id, type=Identity.IdentityType.PASSPORT_NUMBER
    ).get()

    assert registered_passport.value == passport_information


@pytest.mark.django_db
def test_name_update_not_allowed_if_feide_identity_is_present(
    client: APIClient,
    person_foo,
    sponsor_foo,
    unit_foo,
    role_type_foo,
    create_role,
    create_invitation,
    create_invitation_link,
    invitation_valid_date,
):
    # This person has a Feide ID, so the name is assumed to come from there as well
    # and the guest should not be allowed to update it
    role = create_role(
        person=person_foo, sponsor=sponsor_foo, unit=unit_foo, role_type=role_type_foo
    )
    invitation = create_invitation(role)
    invitation_link = create_invitation_link(invitation, invitation_valid_date)
    client.post(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})

    person = invitation_link.invitation.role.person
    data = {
        "person": {
            "first_name": "Someone",
            "last_name": "Test",
            "mobile_phone": "+4797543992",
        }
    }

    # The update request should fail because it contains a name update
    response = client.post(
        reverse("gregui-v1:invited-info"),
        data,
        format="json",
    )
    assert response.status_code == status.HTTP_400_BAD_REQUEST

    # Check that the name did not change in the database
    assert Person.objects.count() == 1
    assert person.first_name == person_foo.first_name
    assert person.last_name == person_foo.last_name


@pytest.mark.django_db
def test_name_update_allowed_if_feide_identity_is_not_present(
    client: APIClient,
    create_person,
    sponsor_foo,
    unit_foo,
    role_type_foo,
    create_role,
    create_invitation,
    create_invitation_link,
    invitation_valid_date,
):
    # This person does not have a Feide ID, so he will be viewed as someone whose name was entered manually
    # by the sponsor, and in it that case the guest can update it
    person = create_person(first_name="Foo", last_name="Bar", email="foo@bar.com")
    role = create_role(
        person=person, sponsor=sponsor_foo, unit=unit_foo, role_type=role_type_foo
    )
    invitation_link = create_invitation_link(
        create_invitation(role), invitation_valid_date
    )
    client.post(reverse("gregui-v1:invite-verify"), data={"uuid": invitation_link.uuid})

    person = invitation_link.invitation.role.person
    data = {
        "person": {
            "first_name": "Someone",
            "last_name": "Test",
            "mobile_phone": "+4797543992",
        }
    }

    response = client.post(
        reverse("gregui-v1:invited-info"),
        data,
        format="json",
    )
    assert response.status_code == status.HTTP_200_OK

    # Check that the name has been updated in the database
    assert Person.objects.count() == 1
    person.refresh_from_db()
    assert person.first_name == "Someone"
    assert person.last_name == "Test"


@pytest.mark.django_db
def test_post_info_fail_fnr_already_verified(client, invited_person_verified_nin):
    """Ensure that an automatically verified fnr cannot be changed"""
    person, invitation_link = invited_person_verified_nin
    fnr = "10093720895"

    session = client.session
    session["invite_id"] = str(invitation_link.uuid)
    session.save()

    url = reverse("gregui-v1:invited-info")
    data = {"person": {"mobile_phone": "+4797543992", "fnr": fnr}}
    response = client.post(url, data, format="json")

    # Verify rejection
    assert response.status_code == status.HTTP_400_BAD_REQUEST
    # Verify fnr was not changed
    person.refresh_from_db()
    assert person.fnr.value != fnr


@pytest.mark.django_db
def test_post_info_fail_unknown_field(client, invited_person_verified_nin):
    """Ensure that including an unknown field gives bad request"""
    _, invitation_link = invited_person_verified_nin

    session = client.session
    session["invite_id"] = str(invitation_link.uuid)
    session.save()

    url = reverse("gregui-v1:invited-info")
    data = {"person": {"mobile_phone": "+4797543992", "badfield": "foo"}}
    response = client.post(url, data, format="json")

    # Verify rejection
    assert response.status_code == status.HTTP_400_BAD_REQUEST


@pytest.mark.django_db
def test_date_of_birth_stored(client, invited_person_verified_nin):
    _, invitation_link = invited_person_verified_nin

    session = client.session
    session["invite_id"] = str(invitation_link.uuid)
    session.save()

    date_of_birth = "1995-10-28"
    url = reverse("gregui-v1:invited-info")
    data = {"person": {"mobile_phone": "+4797543992", "date_of_birth": date_of_birth}}
    response = client.post(url, data, format="json")

    assert response.status_code == status.HTTP_200_OK

    person = Person.objects.get()
    assert (
        person.date_of_birth
        == datetime.datetime.strptime(date_of_birth, "%Y-%m-%d").date()
    )


@pytest.mark.django_db
def test_invalid_date_of_birth_rejected(client, invited_person_verified_nin):
    _, invitation_link = invited_person_verified_nin

    session = client.session
    session["invite_id"] = str(invitation_link.uuid)
    session.save()

    date_of_birth = "1900-03-16"
    url = reverse("gregui-v1:invited-info")
    data = {"person": {"mobile_phone": "+4797543992", "date_of_birth": date_of_birth}}
    response = client.post(url, data, format="json")

    assert response.status_code == status.HTTP_400_BAD_REQUEST

    person = Person.objects.get()
    assert person.date_of_birth is None
